{Modifications of original GeoGet WPT code by pmoravec} 
{Version 0.7}
{Feel free to use or modify}

//Geoget 2

const
  defColorize  = True;
  defShowHint  = True;
  defOffline   = True;
  defOZICE1    = False;
  defCustomURL = '\Storage Card\offline';

var
  counter: integer;
  colorize: boolean;
  showHint: boolean;
  useOffline: boolean;
  forOZICE1: boolean;
  customURL: string;

procedure LoadSettings;
var
  iniF: TIniFile;
begin
  iniF := TIniFile.Create(Geoget_datadir + '\wpt-adv-gg2.ini');
  try
    colorize   := iniF.readBool('General', 'Colorize', defColorize);
    showHint   := iniF.readBool('General', 'ShowHint', defShowHint);
    useOffline := iniF.readBool('Offline', 'Generate', defOffline);
    forOZICE1     := iniF.readBool('Offline', 'OziCE1', defOZICE1);
    customURL  := iniF.readString('Offline', 'CustomURL', defCustomURL);
  finally
    iniF.Free;
  end;
end;

function InitSettings : boolean;
var
  Form: TForm;
  URL: TEdit;
  Color: TCheckBox;
  Hint: TCheckBox;
  Offline : TCheckBox;
  PDA : TCheckBox;
  Button: TButton;
  frmresult : Integer;
begin
 
  Form := TForm.Create(nil);
  Form.Width := 430;
  Form.Height := 220;
  Form.BorderStyle := bsDialog;
  Form.BorderIcons := [];
  Form.Caption := 'Set WPT Export Options';
  Form.Position := poScreenCenter;
  
  Color := TCheckBox.Create(Form);
  Color.Caption := 'Waypoint &colors';
  Color.Checked := colorize;
  Color.Parent  := Form;
  Color.SetBounds(10,10,340,24);

  Hint := TCheckBox.Create(Form);
  Hint.Caption := 'Add &Hints to Waypoint Descriptions';
  Hint.Checked := showHint;
  Hint.Parent  := Form;
  Hint.SetBounds(10,40,340,24);

  Offline := TCheckBox.Create(Form);
  Offline.Checked := useOffline;
  Offline.SetBounds(10,70,400,24);
  Offline.Caption := 'Add Reference to &Offline files copied to:';
  Offline.Parent := Form;

  URL := TEdit.Create(Form);
  URL.Font.Name := 'Tahoma';
  URL.SetBounds(10,100,340,24);
  URL.Parent := Form;
  URL.Text := customURL;

  PDA := TCheckBox.Create(Form);
  PDA.Checked := forOZICE1;
  PDA.SetBounds(10,130,400,24);
  PDA.Caption := 'OziCE &1.x Export to "Data File Path" subdirectory ---^ ';
  PDA.Parent := Form;

  Button := TButton.Create(Form);
  Button.Left := 30;
  Button.Top := 160;
  Button.Width := 80;
  Button.Height := 24;
  Button.Caption := '&Apply';
  Button.Parent := Form;
  Button.Default := True;
  Button.Modalresult := mrOk;

  Button := TButton.Create(Form);
  Button.Left := 120;
  Button.Top := 160;
  Button.Width := 110;
  Button.Height := 24;
  Button.Caption := 'Apply + &Save';
  Button.Parent := Form;
  Button.Modalresult := mrYes;

  Button := TButton.Create(Form);
  Button.Left := 240;
  Button.Top := 160;
  Button.Width := 130;
  Button.Height := 24;
  Button.Caption := 'Use O&ld Settings';
  Button.Cancel := True;
  Button.Parent := Form;
  Button.Modalresult := mrCancel;

  frmresult := form.showmodal;
  if frmresult =  mrYes  then
    Result := True
  else
   Result:= False;
  if (frmresult = mrYes) or (frmresult = mrOk) then
    begin
      customURL := URL.text;
      colorize  := Color.checked;
      showHint  := Hint.checked; 
      useOffline:= Offline.checked;
      forOZICE1    := PDA.checked;
    end
  else
    LoadSettings;
  if useOffline and ((forOZICE1) xor ((customURL[1] <> '/') and (customURL[1] <> '\'))) then
  begin
     if (forOZICE1) then 
       ShowMessage('You must use relative path (starting at OZI CE "Data file path" set in "Map and Data Files") for OZI_CE 1.x!' + CRLF + 'Please correct your offline data directory now (most likely to ''offline'') and click on ''Apply && Save''.')
     else
       ShowMessage('You must use absolute path starting with \ in OziCE 2.x or C:\ in OZIExplorer!' + CRLF + 'Please correct your offline data directory now (most likely to ''\Storage Card\offline'') and click on ''Apply && Save''.');
     Result := InitSettings;
  end;
end;


procedure SaveSettings;
var
  iniF: TIniFile;
begin
  iniF := TIniFile.Create(Geoget_datadir + '\wpt-adv-gg2.ini');
  try
    iniF.writeBool('General', 'Colorize', colorize);
    iniF.writeBool('General', 'ShowHint', showHint);
    iniF.writeBool('Offline', 'Generate', useOffline);
    iniF.writeBool('Offline', 'OziCE1', forOZICE1);
    iniF.writeString('Offline', 'CustomURL', customURL );
  finally
    iniF.Free;
  end;
end;


function ExportExtension: string;
begin
  result := 'WPT';
end;

function ExportDescription: string;
begin
  result := 'OziExp WPT File with Advanced Options';
end;

function ExportHeader: string;
begin
  LoadSettings;
  if InitSettings then SaveSettings;
  Result := 'OziExplorer Waypoint File Version 1.1' + CRLF;
  Result := Result + 'WGS 84' + CRLF;
  Result := Result + 'Reserved 2' + CRLF;
  Result := Result + 'Reserved 3' + CRLF;
  counter := 1;
end;

function ExportFooter: string;
begin
  if forOZICE1 and (counter > 1000) then
    MessBox(inttostr(counter) + ' points were exported, but only 1000 of them will be accessible in OZICE 1.x! Try to reduce the number of points in your selection.', 'Warning', 0);
  result := '';
end;

function ColorPoint: string;
var
  color: string;
  bgcolor: string;
begin
  if colorize then 
  begin
	  color := '0'; 			    {Default text is Black}
	  bgcolor := '65535';                       {Unlisted type - Black on Yellow}
	  if length(GC.IDTag) = 4 then		    {Is the cache code present?}
	  case Ord(GC.IDTag[1]) of		    {--if so, use the first letter value}
		 69 : bgcolor := '9869006';         {Event - pink from Event icon} 
		 71 : bgcolor := '14734496';        {EarthCache - Aqua} 
		 72 : bgcolor := '16575206';        {LetterBox Hybrid - Envelope icon color}
		 73 : bgcolor := '9441424';         {WhereIGo - Blue from WIG icon}
		 77 : begin
			 color   := '16777215';     {Multi} 
			 bgcolor := '827634';       {White on Average of orange box colors}
		      end;
		 84 : begin
			 color   := '16777215';     {Traditional}
			 bgcolor := '3901272';      {White on avg. of Green box colors}
		      end;
		 85 : begin
			 color   := '16777215';
			 bgcolor := '12657920';     {Unknown - White on Dark Blue}
		      end;
		 86 : bgcolor := '16575206';        {Virtual - 1/3 Gray}
		 87 : bgcolor := '7500402';         {Webcam - 55% Gray}	 
	  end;	 
	
	  if GC.IsOwner then
	  begin
	    {color := bgcolor;}
	    bgcolor := '15631086'; 
	  end;

	  if GC.IsFound then                 {Found}
	  begin
	    color := bgcolor;
	    bgcolor := '12382140';	     {Cache color on Green (from cache list)}
	  end;

	  if GC.IsArchived then              {Archived}
	  begin
	    color := bgcolor;		
	    bgcolor := '255';	             {Cache Color on Light Red Background}
	  end
	  else if GC.IsDisabled then
	  begin
	    color := bgcolor;
	    bgcolor := '3355647';	     {Cache Color on Red Background}
	  end;
       Result := color + ',' + bgcolor;
  end
  else
    Result := '0,65535';
end;

function ColorWaypoint(wpt: TWpt): string;
var
  color: string;
  bgcolor: string;
begin
  color     := '0';                      {Black text by default}
  bgcolor   := '65535';                  {Yellow background}

  if colorize then
  begin
	  if wpt.IsFinal then
	  begin
	    color   := '16777215';
	    bgcolor := '57504';
	  end
	  else if wpt.IsUserWaypoint then
	  begin
	    color   := '8388608';              {Blue on Yellow Background}
	    bgcolor := '65535';
	  end	    
	  else if GC.IsOwner then
	  begin
	    {color := bgcolor;}
	    bgcolor := '15646958'; 
	  end;
	 
	  if GC.IsFound then
	  begin
	    color   := '32768';                {Green on Yellow background}
	    bgcolor := '10092543';          {Yellow Background (from cache list)}
	  end	    
	  else if GC.IsDisabled or GC.IsArchived then
	  begin
	    color := bgcolor;
	    bgcolor := '5263615';	     {Cache Color on Light Red Background}
	  end;
  end;
  Result := color + ',' + bgcolor;
end;

function ExportPoint: string;
var
  s: string;
  hint : string;
  attachment: string;
  n : integer;
begin
  s := IntToStr(Counter) + ',';
  Counter := Counter + 1;
  s := s + GC.ID + ' ';
  hint := '';
  attachment := '';

  {Prepare the hint}
  if showHint then
    hint := ReplaceString(ReplaceString(ReplaceString(ReplaceString(UtfToAscii(GC.Hint), Chr($0A), ''), Chr($0D), ' ' ), ',', ';'), '<br>', ' ');

  {Expecting copy of 'offline' folder in OziExplorer's data directory}
  if useOffline and (length(customURL) > 0) then
    attachment := ',,,,';
    attachment := attachment + customURL + '/cache/' + GC.ID[length(GC.ID)] + 
                                                          '/' + GC.ID + '.htm' + ',,';
    if forOZICE1 then
      attachment := ReplaceString(attachment, '\', '/')
    else
      attachment := ReplaceString(attachment, '/', '\');

  s := s + + GC.IDTag + ' ' + ReplaceString(UtfToAscii(GC.Name), ',', ';') + ',';
  s := s + GC.Lat + ',' + GC.Lon + ',';
  s := s + '25569.00000,0,1,3,' + ColorPoint + ',' + hint + ',0,0,0,-777,6,0,17' + attachment;
  Result := s + CRLF;
  for n := 0 to GC.Waypoints.Count - 1 do
  begin
    if GC.Waypoints[n].IsEmptyCoord then continue;
    s := IntToStr(Counter) + ',';
    Counter := Counter + 1;
    s := s + GC.ID + ' ' + GC.IDTag + ' ' + ReplaceString(UtfToAscii(GC.Waypoints[n].FullName), ',', ';') + ',';
    s := s + GC.Waypoints[n].Lat + ',' + GC.Waypoints[n].Lon + ',';
    s := s + '25569.00000,0,1,3,' + ColorWaypoint(GC.Waypoints[n]) + ',';
    if GC.Waypoints[n].IsFinal then 
       s := s + hint + ':' + ReplaceString(UtfToAscii(GC.Waypoints[n].Comment), ',', ';')
    else 
       s := s + ReplaceString(UtfToAscii(GC.Waypoints[n].Description), ',', ';'); 
    s := s + ',0,0,0,-777,6,0,17' + attachment;
    Result := result + s + CRLF;
  end;
end;
